<?php

namespace B2buy\SupplierMaster\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use B2buy\SupplierMaster\Http\Requests\PolicyRequest;
use B2buy\SupplierMaster\Interfaces\PolicyRepositoryInterface;
use B2buy\SupplierMaster\Models\Policy;

/**
 * Resource controller class for policy.
 */
class PolicyResourceController extends BaseController
{

    /**
     * Initialize policy resource controller.
     *
     * @param type PolicyRepositoryInterface $policy
     *
     * @return null
     */
    public function __construct(PolicyRepositoryInterface $policy)
    {
        parent::__construct();
        $this->repository = $policy;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\B2buy\SupplierMaster\Repositories\Criteria\PolicyResourceCriteria::class);
    }

    /**
     * Display a list of policy.
     *
     * @return Response
     */
    public function index(PolicyRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\B2buy\SupplierMaster\Repositories\Presenter\PolicyPresenter::class)
                ->$function();
        }

        $policies = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('supplier_master::policy.names'))
            ->view('supplier_master::policy.index', true)
            ->data(compact('policies', 'view'))
            ->output();
    }

    /**
     * Display policy.
     *
     * @param Request $request
     * @param Model   $policy
     *
     * @return Response
     */
    public function show(PolicyRequest $request, Policy $policy)
    {

        if ($policy->exists) {
            $view = 'supplier_master::policy.show';
        } else {
            $view = 'supplier_master::policy.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('supplier_master::policy.name'))
            ->data(compact('policy'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new policy.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PolicyRequest $request)
    {

        $policy = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('supplier_master::policy.name')) 
            ->view('supplier_master::policy.create', true) 
            ->data(compact('policy'))
            ->output();
    }

    /**
     * Create new policy.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PolicyRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $policy                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('supplier_master::policy.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('supplier_master/policy/' . $policy->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/supplier_master/policy'))
                ->redirect();
        }

    }

    /**
     * Show policy for editing.
     *
     * @param Request $request
     * @param Model   $policy
     *
     * @return Response
     */
    public function edit(PolicyRequest $request, Policy $policy)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('supplier_master::policy.name'))
            ->view('supplier_master::policy.edit', true)
            ->data(compact('policy'))
            ->output();
    }

    /**
     * Update the policy.
     *
     * @param Request $request
     * @param Model   $policy
     *
     * @return Response
     */
    public function update(PolicyRequest $request, Policy $policy)
    {
        try {
            $attributes = $request->all();

            $policy->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('supplier_master::policy.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('supplier_master/policy/' . $policy->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('supplier_master/policy/' . $policy->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the policy.
     *
     * @param Model   $policy
     *
     * @return Response
     */
    public function destroy(PolicyRequest $request, Policy $policy)
    {
        try {

            $policy->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('supplier_master::policy.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('supplier_master/policy/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('supplier_master/policy/' . $policy->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple policy.
     *
     * @param Model   $policy
     *
     * @return Response
     */
    public function delete(PolicyRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('supplier_master::policy.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('supplier_master/policy'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/supplier_master/policy'))
                ->redirect();
        }

    }

    /**
     * Restore deleted policies.
     *
     * @param Model   $policy
     *
     * @return Response
     */
    public function restore(PolicyRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('supplier_master::policy.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/supplier_master/policy'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/supplier_master/policy/'))
                ->redirect();
        }

    }

}
